import asyncio
import csv
import sys

from aiohttp.client import ClientSession

from config import settings
from skyhub.utils.telegram_erros import send_message

from .controllers import BlingController, SkyhubController
from .loggin_config import *
from .models.produto import Produto


async def main():
    async with ClientSession() as session:
        headers_skyhub = {
            "X-User-Email": settings.user_email,
            "X-Api-Key": settings.api_key,
            "x-accountmanager-key": settings.erp,
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        skyhub_controller = SkyhubController(
            credentials=headers_skyhub,
            base_url=settings.url_skyhub,
            request=session,
        )

        bling_controller = BlingController(
            credentials={"apikey": settings.bling_key},
            base_url=settings.url_bling,
            request=session,
        )

        with open(settings.file_path, "r", encoding="utf-8") as f:
            csv_file = csv.reader(f, delimiter=";")

            for index, row in enumerate(csv_file):
                if index == 0 or row[1] == "":
                    continue

                produto_estoque = Produto(
                    row[0], row[1].replace(",", "."), row[2].replace(",", ".")
                )

                produto_skyhub = await skyhub_controller.get_produto(str(row[0]))
                produto_bling = await bling_controller.get_produto(str(row[0]))

                if produto_skyhub:
                    print(index, produto_skyhub)
                    if produto_skyhub.price != row[1] or produto_skyhub.qty != row[2]:
                        await skyhub_controller.atualiza_produto(produto_estoque)

                if produto_bling:
                    print(index, produto_bling)
                    if produto_bling.price != row[1] or produto_bling.qty != row[2]:
                        await bling_controller.atualiza_produto(produto_estoque)


if __name__ == "__main__":
    try:
        asyncio.run(main())

    except KeyboardInterrupt:
        sys.exit()

    except Exception as e:
        send_message(e)



"""Envia uma menssagem para o chat do telegram quando ocorre um erro no sistema."""
import logging

import requests

from config import settings
from skyhub.loggin_config import *

logger = logging.getLogger(__name__)
logger.addHandler(f_handler)


def send_message(message):
    token = settings.bot_token
    url = f"{settings.url_telgram}{token}/sendMessage"
    params = {
        "chat_id": "325105532",
        "text": f'<code class="language-python">{message}</code>',
        "parse_mode": "html",
    }

    response = requests.get(url, params=params)

    status = response.status_code

    if status == 200:
        logger.info("Menssagem de erro enviada.")

    else:
        logger.error("A menssagem não foi enviada")



import logging

from aiohttp import ClientSession

from skyhub.loggin_config import *
from skyhub.models.produto import Produto

logger = logging.getLogger(__name__)
logger.addHandler(f_handler)


class BlingController:
    def __init__(
        self, credentials: dict, base_url: str, request: ClientSession
    ) -> None:
        self.credentials = credentials
        self.base_url = base_url
        self.request = request

    async def get_produto(self, sku: str) -> Produto:
        url = f"{self.base_url}/{sku}/json?estoque=S"
        response = await self.request.request(
            method="GET", url=url, params=self.credentials
        )
        response_json = await response.json()
        retorno = response_json.get("retorno")

        if "erros" in retorno:
            logger.warning(f"O produto {sku} não foi encontrado!")
            return False

        if response.status == 200:
            produtos = retorno.get("produtos")
            produtoJson = produtos[0].get("produto")

            codigo = produtoJson.get("codigo")
            preco = produtoJson.get("preco")
            estoque_atual = produtoJson.get("estoqueAtual")

            return Produto(codigo, preco, estoque_atual)

    async def atualiza_produto(self, produto: Produto) -> bool:
        xml = produto.toXML()
        apikey = self.credentials.get("apikey", "")
        data = {"apikey": apikey, "xml": xml}

        url = f"{self.base_url}/{produto.sku}/json?estoque=S"
        response = await self.request.request(method="POST", url=url, data=data)

        if response.status == 201:
            logger.info(f"O produto '{produto.sku}' foi atualizado.")
            return True

        else:
            print(response.status)
            logger.warning(f"O produto '{produto.sku}' não pode ser atualizado.")
            return False
